classdef DipolarGas < handle & matlab.mixin.Copyable

    properties (Access = public)
        SimulationMode; 
        TimeStep;
        SimulationTime;
        NumberOfAtoms;
        
        % Etol    = 5e-10; % Tolerances
        % rtol    = 1e-5;
        % cut_off = 2e6;   % sometimes the imaginary time gets a little stuck
                         % even though the solution is good, this just stops it going on forever
        % mindt = 1e-6; %Minimum size for a time step using adaptive dt

        %Flags
        
        
        DebugMode;
        DoSave;
        SaveDirectory;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %- Methods
    
    methods
        function this  = DipolarGas(varargin)
            
            p = inputParser;
            p.KeepUnmatched = true;
            addParameter(p, 'SimulationMode', 'ImaginaryTimeEvolution',... 
                @(x) any(strcmpi(x,{'ImaginaryTimeEvolution','RealTimeEvolution'})));
            addParameter(p, 'NumberOfAtoms',    5000,...
                @(x) assert(isnumeric(x) && isscalar(x) && (x > 0)));  
            addParameter(p, 'TimeStep',       0.0005,...
                @(x) assert(isnumeric(x) && isscalar(x) && (x > 0)));  
            addParameter(p, 'SimulationTime',       3e-03,...
                @(x) assert(isnumeric(x) && isscalar(x) && (x > 0)));  
            addParameter(p, 'DebugMode',            false,...
                @islogical);
            addParameter(p, 'SaveData',             false,...
                @islogical);
            addParameter(p, 'SaveDirectory',     './Data',...
                @ischar);
            
            p.parse(varargin{:});
            
            this.SimulationMode       = p.Results.SimulationMode;
            this.ErrorEstimationMethod= p.Results.ErrorEstimationMethod;
            
            this.NumberOfAtoms        = p.Results.NumberOfAtoms;
            this.TimeStep             = p.Results.TimeStep;
            this.SimulationTime       = p.Results.SimulationTime;  
            
            this.DebugMode            = p.Results.DebugMode;
            this.DoSave               = p.Results.SaveData;
            this.SaveDirectory        = p.Results.SaveDirectory;
            
            switch this.SimulationMode
                case "ImaginaryTimeEvolution" 
                % Development In progress
                case "RealTimeEvolution"
                % Development In progress
            end
        end
    end

    methods
        function set.TimeStep(this, val)
            assert(val > 1e-06, 'Not time efficient to compute for time steps smaller than 1 microsecond!');
            this.TimeStep = val;
        end
        function ret = get.TimeStep(this)
            ret = this.TimeStep;
        end
        function set.SimulationTime(this, val)
%             assert(val <= 5e-03, 'Not time efficient to compute for time spans longer than 5 milliseconds!');
            this.SimulationTime = val;
        end
        function ret = get.SimulationTime(this)
            ret = this.SimulationTime;
        end
        function set.NumberOfAtoms(this, val)
            assert(val <= 50000, '!!Not time efficient to compute for atom numbers larger than 50,000!!');
            this.NumberOfAtoms = val;
        end
        function ret = get.NumberOfAtoms(this)
            ret = this.NumberOfAtoms;
        end
        
        function set.DebugMode(this, val)
            this.DebugMode = val;
        end
        function ret = get.DebugMode(this)
            ret = this.DebugMode;
        end
        function set.DoSave(this, val)
            this.DoSave = val;
        end
        function ret = get.DoSave(this)
            ret = this.DoSave;
        end
        function set.SaveDirectory(this, val)
            this.SaveDirectory = val;
        end
        function ret = get.SaveDirectory(this)
            ret = this.SaveDirectory;
        end
        
    end % - setters and getters
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %- Methods
    
    methods(Access = protected)
        function cp = copyElement(this)
            % Shallow copy object
            cp = copyElement@matlab.mixin.Copyable(this);
            
            % Forces the setter to redefine the function handles to the new copied object
            
            pl = properties(this);
            for k = 1:length(pl)
                sc = superclasses(this.(pl{k}));
                if any(contains(sc,{'matlab.mixin.Copyable'}))
                    cp.(pl{k}) = this.(pl{k}).copy();
                end
            end
        end
    end
    
    methods (Static)
        
        % Creates an Instance of Class, ensures singleton behaviour (that there
        % can only be one Instance of this class
        function singleObj = getInstance(varargin)
            % Creates an Instance of Class, ensures singleton behaviour
            persistent localObj;
            if isempty(localObj) || ~isvalid(localObj)
                localObj =  Simulator.DipolarGas(varargin{:});
            end
            singleObj = localObj;
        end
    end
    
end
